//+------------------------------------------------------------------+
//|                                      Williams Percent Range.mq4 |
//|                      Copyright  2005, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#property copyright "Copyright  2005, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net/"
//----
#property indicator_separate_window
#property indicator_minimum -100
#property indicator_maximum 0
#property indicator_buffers 1
#property indicator_color1 DodgerBlue
#property indicator_level1 -20
#property indicator_level2 -80
#property indicator_level3 -50
//---- input parameters
extern int ExtWPRPeriod = 14;
extern string note = "turn on Alert = true; turn off = false";
extern bool AlertOn = true;
extern int       Level1=-20;
extern int       Level2=-50;
extern int       Level3=-80;

//---- buffers
double ExtWPRBuffer[];
bool   AlertBuy1  = false;
bool   AlertSell1 = false;
bool   AlertBuy2  = false;
bool   AlertSell2 = false;
bool   AlertBuy3  = false;
bool   AlertSell3 = false;
// Show regular timeframe string (HCY)
string AlertPrefix;
string GetTimeFrameStr() {
   switch(Period())
   {
      case 1 : string TimeFrameStr="M1"; break;
      case 5 : TimeFrameStr="M5"; break;
      case 15 : TimeFrameStr="M15"; break;
      case 30 : TimeFrameStr="M30"; break;
      case 60 : TimeFrameStr="H1"; break;
      case 240 : TimeFrameStr="H4"; break;
      case 1440 : TimeFrameStr="D1"; break;
      case 10080 : TimeFrameStr="W1"; break;
      case 43200 : TimeFrameStr="MN1"; break;
      default : TimeFrameStr="CUR";
   } 
   return (TimeFrameStr);
   }
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   string sShortName;
//---- indicator buffer mapping
   SetIndexBuffer(0, ExtWPRBuffer);
//---- indicator line
   SetIndexStyle(0, DRAW_LINE);
//---- name for DataWindow and indicator subwindow label
   sShortName="%R(" + ExtWPRPeriod + ")";
   IndicatorShortName(sShortName);
   SetIndexLabel(0, sShortName);
//---- first values aren't drawn
   SetIndexDrawBegin(0, ExtWPRPeriod);
//----
   AlertPrefix=Symbol()+" ("+GetTimeFrameStr()+"):  ";
   return(0);
  }
//+------------------------------------------------------------------+
//| Williams Percent Range                                          |
//+------------------------------------------------------------------+
int start()
  {
   int i, nLimit, nCountedBars;  
//---- insufficient data
   if(Bars <= ExtWPRPeriod) 
       return(0);
//---- bars count that does not changed after last indicator launch.
   nCountedBars = IndicatorCounted();
//----Williams Percent Range calculation
   i = Bars - ExtWPRPeriod - 1;
   if(nCountedBars > ExtWPRPeriod) 
       i = Bars - nCountedBars - 1;  
   while(i >= 0)
     {
       double dMaxHigh = High[Highest(NULL, 0, MODE_HIGH, ExtWPRPeriod, i)];
       double dMinLow = Low[Lowest(NULL, 0, MODE_LOW, ExtWPRPeriod, i)];      
       if(!CompareDouble((dMaxHigh - dMinLow), 0.0))
           ExtWPRBuffer[i] = -100*(dMaxHigh - Close[i]) / (dMaxHigh - dMinLow);
       i--;
     }
//----
   // ======= Alert =========
   if(AlertOn && NewBar()){
      showAlert();
      }
   // ======= Alert Ends =========
   return(0);
  }
//+------------------------------------------------------------------+
//|     .                        |
//+------------------------------------------------------------------+
bool CompareDouble(double Number1, double Number2)
  {
    bool Compare = NormalizeDouble(Number1 - Number2, 8) == 0;
    return(Compare);
  } 
//+------------------------------------------------------------------+ 
bool NewBar()
{
   static datetime lastbar;
   datetime curbar = Time[0];
   //Print("NewBar(). lastbar="+TimeToStr(lastbar,TIME_DATE|TIME_MINUTES)+"  curbar="+TimeToStr(curbar,TIME_DATE|TIME_MINUTES));
   if(lastbar!=curbar)
   {
      lastbar=curbar;
      return (true);
   }
   else
   {
      return(false);
   }
} 


void showAlert(){
   string s="";
   if(NewBar()){
      AlertSell1 = false; 
      AlertBuy1  = false;
      AlertSell2 = false; 
      AlertBuy2 = false;
      AlertSell3 = false; 
      AlertBuy3 = false;
   }

  
   if (ExtWPRBuffer[0]>Level1 && ExtWPRBuffer[1]<=Level1 && AlertBuy1==false){
      
      PlaySound("expert.wav");
      Alert(AlertPrefix+" %R("+ExtWPRPeriod+") UP cross "+Level1+ " "+ExtWPRBuffer[0]);
      AlertBuy1  = true;
      AlertSell1 = false;
   }
   if (ExtWPRBuffer[0]<Level1 && ExtWPRBuffer[1]>=Level1 && AlertSell1==false){
      
      PlaySound("expert.wav");
      Alert(AlertPrefix+" %R("+ExtWPRPeriod+") Down cross "+Level1+ " "+ExtWPRBuffer[0]);
      AlertBuy1  = false;
      AlertSell1 = true;
   }
   
   
   if (ExtWPRBuffer[0]>Level2 && ExtWPRBuffer[1]<=Level2 && AlertBuy2==false){
      
      PlaySound("expert.wav");
      Alert(AlertPrefix+" %R("+ExtWPRPeriod+") UP cross "+Level2+ " "+ExtWPRBuffer[0]);
      AlertBuy2  = true;
      AlertSell2 = false;
   }   
   if (ExtWPRBuffer[0]<Level2 && ExtWPRBuffer[1]>=Level2 && AlertSell2==false){
      
      PlaySound("expert.wav");
      Alert(AlertPrefix+" %R("+ExtWPRPeriod+") Down cross "+Level2+ " "+ExtWPRBuffer[0]);
      AlertBuy2  = false;
      AlertSell2 = true;
   } 
   
    
   if (ExtWPRBuffer[0]>Level3 && ExtWPRBuffer[1]<=Level3 && AlertBuy3==false){
      
      PlaySound("expert.wav");
      Alert(AlertPrefix+" %R("+ExtWPRPeriod+") UP cross "+Level3+ " "+ExtWPRBuffer[0]);
      AlertBuy3  = true;
      AlertSell3 = false;
   }   
   if (ExtWPRBuffer[0]<Level3 && ExtWPRBuffer[1]>=Level3 && AlertSell3==false){
      PlaySound("expert.wav");
      Alert(AlertPrefix+" %R("+ExtWPRPeriod+") Down cross "+Level3+ " "+ExtWPRBuffer[0]);
      AlertBuy3  = false;
      AlertSell3 = true;
   }
}